@@ -6,14 +6,14 @@ import os
 import datetime
 from typing import List, Optional, Any, Dict, Sequence, Type
 
-from graphviz import Digraph
 from mixins import ReprMixin
 
 from datacode.graph.base import GraphObject, Graphable, GraphFunction
 from datacode.graph.edge import Edge
+from datacode.models.dethash import DeterministicHashDictMixin
 from datacode.graph.node import Node
 from datacode.logger import logger
-from datacode.models.links import LinkedItem
+from datacode.models.last_modified import LinkedLastModifiedItem, most_recent_last_modified
 from datacode.models.outputter import DataOutputter
 from datacode.models.types import SourceCreatingPipeline
 from datacode.summarize import describe_df
@@ -24,7 +24,7 @@ from datacode.models.loader import DataLoader
 import datacode.hooks as hooks
 
 
-class DataSource(LinkedItem, Graphable, ReprMixin):
+class DataSource(LinkedLastModifiedItem, Graphable, DeterministicHashDictMixin, ReprMixin):
     copy_keys = [
         'location',
         'name',
@@ -154,7 +154,6 @@ class DataSource(LinkedItem, Graphable, ReprMixin):
                 raise ValueError(f'variable name {name} repeated in load variables')
             existing_names.append(name)
 
-
     @property
     def df(self):
         if self._df is None:
@@ -181,13 +180,6 @@ class DataSource(LinkedItem, Graphable, ReprMixin):
     def last_modified(self, value: Optional[datetime.datetime]):
         self._last_modified = value
 
-    @property
-    def pipeline_last_modified(self):
-        if self.pipeline is None:
-            return None
-
-        return self.pipeline.last_modified
-
     @property
     def pipeline(self) -> Optional[SourceCreatingPipeline]:
         return self._pipeline
@@ -199,6 +191,12 @@ class DataSource(LinkedItem, Graphable, ReprMixin):
             self._add_back_link(item)
             item._add_forward_link(self)
 
+    @property
+    def __dict__(self):
+        if hasattr(self, '_df'):
+            return {**super().__dict__, 'last_modified': self.last_modified}
+        return super().__dict__
+
     def _load(self):
         logger.debug(f'Started loading source {self}')
         hooks.on_begin_load_source(self)
@@ -235,6 +233,15 @@ class DataSource(LinkedItem, Graphable, ReprMixin):
         pass
         # assert not (filepath is None) and (df is None)
 
+    @property
+    def should_run_pipeline(self) -> bool:
+        lm = most_recent_last_modified(self.last_modified, self.pipeline_last_modified)
+        if lm is None:
+            return True
+        if lm == self.last_modified:
+            return False
+        return True
+
     def _set_data_loader(self, data_loader_class: Type[DataLoader], pipeline: SourceCreatingPipeline = None,
                          **read_file_kwargs):
         logger.debug(f'Setting data loader for source {self.name}')
@@ -243,39 +250,43 @@ class DataSource(LinkedItem, Graphable, ReprMixin):
             # if a source in the pipeline to create this data source was modified more recently than this data source
             # note: if there is no location, will always enter the next block, as last modified time will set
             # to a long time ago
-            if (
-                    # no existing location for this source, must use pipeline
-                    self.last_modified is None or
-                    # not able to determine when pipeline sources were modified, must always run pipeline
-                    pipeline.last_modified is None or
-                    # pipeline sources were modified more recently than this source, run pipeline
-                    pipeline.last_modified > self.last_modified
-            ):
-                # a prior source used to construct this data source has changed. need to re run pipeline
+            source_lm = self.last_modified
+            pipeline_lm = self.pipeline_last_modified
+            lm = most_recent_last_modified(source_lm, pipeline_lm)
+            if lm is None:
                 run_pipeline = True
-                if pipeline.last_modified is None:
-                    logger.warning(f"""
-                    Was not able to determine last modified of pipeline {pipeline.name}.
-                    Will always run pipeline due to this. Consider manually setting last_modified when creating
-                    the pipeline.
-                    """.strip())
-                elif self.last_modified is None:
-                    logger.warning(f"""
-                   Was not able to determine last modified of source {self.name}.
-                   Will run pipeline due to this. This is due to no file currently existing for this source.
-                   """.strip())
+            elif lm == pipeline_lm and lm != source_lm:
+                run_pipeline = True
+
+            if run_pipeline:
+                # a prior source used to construct this data source has changed. need to re run pipeline
+                if pipeline_lm is None:
+                    logger.warning(
+                        f"Was not able to determine last modified of pipeline "
+                        f"{pipeline.name}. Will always run pipeline due to this. "
+                        f"Consider manually setting last_modified when creating "
+                        f"the pipeline."
+                    )
+                elif source_lm is None:
+                    logger.warning(
+                        f"Was not able to determine last modified of source "
+                        f"{self.name}. Will run pipeline due to this. "
+                        f"This is due to no file currently existing for this source."
+                    )
                 else:
-                    recent_obj = pipeline.obj_last_modified
+                    recent_obj, obj_lm = self.pipeline_obj_last_modified
                     try:
                         recent_obj_name = recent_obj.name
                     except AttributeError:
                         # Must be Operation, get name from pipeline instead
                         recent_obj_name = pipeline.name
-                    logger.warning(f'''{recent_obj_name} was modified at {recent_obj.last_modified}.
-
-                    this data source {self.name} was modified at {self.last_modified}.
-
-                    to get new changes, will load this data source through pipeline rather than from file.''')
+                    logger.warning(
+                        f'{recent_obj_name} was modified at {obj_lm}. '
+                        f'This data source {self.name} was modified at '
+                        f'{self.last_modified}. To get new changes, '
+                        f'will load this data source through pipeline '
+                        f'rather than from file.'
+                    )
 
             # otherwise, don't need to worry about pipeline, continue handling
 
@@ -650,6 +661,13 @@ class DataSource(LinkedItem, Graphable, ReprMixin):
             elems.append(Edge(self.pipeline.primary_node(include_attrs, func_dict), pn))
         return elems
 
+    @property
+    def cache_json_location(self) -> Optional[str]:
+        if self.location is None:
+            return None
+
+        return f'{self.location}.dcc.json'
+
 
 class NoColumnForVariableException(Exception):
     pass
